"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = require("node:path");
const node_url_1 = require("node:url");
const clone = require("clone");
const fs_extra_1 = require("fs-extra");
const qs_1 = require("qs");
const browser_1 = require("../browser");
const server_1 = require("../server");
class FacadePuppeteer {
    static create(options) {
        return new FacadePuppeteer(options);
    }
    constructor(options = {}) {
        const { stopImageRequests = false } = options;
        this.stopImageRequests = stopImageRequests;
    }
    async createPage(url, requirePuppeteerCb, waitForOptions, rendererOptions = {}, browserUrl) {
        const { pageUrl, server } = await this.hostContent(url);
        const puppeteer = await requirePuppeteerCb();
        const { browser, closeBrowser } = await (0, browser_1.createBrowser)(puppeteer, rendererOptions, browserUrl);
        this.browser = browser;
        this.closeBrowser = async () => {
            await closeBrowser();
            server === null || server === void 0 ? void 0 : server.close();
        };
        this.page = await this.browser.newPage();
        await this.exposeClone();
        this.stopImageRequests && await this.blockImageRequests();
        await this.page.goto(pageUrl, waitForOptions);
    }
    async hostContent(url) {
        const { href, protocol } = new URL(url);
        if (protocol === 'file:') {
            const port = await (0, server_1.getFreePort)();
            const rootPath = (0, node_path_1.dirname)((0, node_url_1.fileURLToPath)(href));
            const server = await (0, server_1.createServer)(rootPath, port);
            const pageUrl = `http://localhost:${port}`;
            return { pageUrl, server };
        }
        return { pageUrl: url };
    }
    async exposeClone() {
        try {
            await this.page.exposeFunction('clone', clone);
        }
        catch (e) {
            console.log('\'Clone\' function already exposed!');
        }
    }
    async addScriptsToPage(scripts) {
        for (const script of scripts) {
            await this.page.addScriptTag({ path: script });
        }
    }
    async screenshot(opts) {
        await this.page.screenshot(opts);
    }
    async createRequestInterceptor(opts, requestedFile) {
        this.page.removeAllListeners('request');
        await this.page.setRequestInterception(true);
        this.page.on('request', (request) => {
            if (request.url().includes(requestedFile)) {
                const { theme, clm } = opts;
                const queryString = (0, qs_1.stringify)({ theme, clm }, { addQueryPrefix: true });
                request.continue({ url: `${request.url()}${queryString}` });
                return;
            }
            request.continue();
        });
        await this.page.reload();
    }
    async renderBlocks(contentPath, blocks = []) {
        const renderOneBlock = blocks.length === 1;
        const editorJS = (0, node_path_1.join)(contentPath, 'build/dev/editor.js');
        const renderBlocksJS = (0, node_path_1.join)(__dirname, '../../injected-scripts/renderBlocks.js');
        const injectedScriptsArray = [renderBlocksJS];
        const packageJSON = await (0, fs_extra_1.readJSON)((0, node_path_1.join)(contentPath, 'package.json'));
        if (packageJSON.dependencies.ewizardjs) {
            injectedScriptsArray.push(editorJS);
        }
        await this.addScriptsToPage(injectedScriptsArray);
        await this.page.evaluate(({ renderOneBlock, blocks }) => {
            if (renderOneBlock) {
                const block = window.__blocks.find(({ name }) => blocks.includes(name));
                window.__renderBlock(block);
            }
            else {
                const currentThemeBlocks = window.__blocks.filter(({ name }) => blocks.includes(name));
                const blocksForRender = currentThemeBlocks.length ? currentThemeBlocks : window.__blocks;
                window.__renderBlocks(blocksForRender);
            }
        }, { renderOneBlock, blocks });
    }
    async getWindowData(windowProps) {
        const propsObj = {};
        const context = { propsObj, windowProps };
        return this.page.evaluate(async (context) => {
            for (const prop of context.windowProps) {
                context.propsObj[prop] = await clone(window[prop]);
            }
            return context.propsObj;
        }, context);
    }
    async getElement(selector) {
        return this.page.$(selector);
    }
    async blockImageRequests() {
        await this.page.setRequestInterception(true);
        this.page.on('request', (req) => {
            return (req.resourceType() === 'image')
                ? req.abort()
                : req.continue();
        });
    }
}
exports.default = FacadePuppeteer;
//# sourceMappingURL=index.js.map